@@ -49,6 +49,14 @@ fn diff_recursive<'a, 'b>(
             new_node: new,
         });
 
+        if let Some(velem) = old.as_velement_ref() {
+            if velem.special_attributes.on_remove_element_key().is_some() {
+                patches.push(Patch::SpecialAttribute(
+                    PatchSpecialAttribute::CallOnRemoveElem(*old_node_idx, old),
+                ));
+            }
+        }
+
         if let VirtualNode::Element(new_element_node) = new {
             for child in new_element_node.children.iter() {
                 increment_idx_for_child(child, new_node_idx);
@@ -136,15 +144,15 @@ fn diff_recursive<'a, 'b>(
 
             // FIXME: Move into function
             match (
-                old_element.special_attributes.on_create_elem.as_ref(),
-                new_element.special_attributes.on_create_elem.as_ref(),
+                old_element.special_attributes.on_create_element_key(),
+                new_element.special_attributes.on_create_element_key(),
             ) {
                 (None, Some(_)) => {
                     patches.push(Patch::SpecialAttribute(
                         PatchSpecialAttribute::CallOnCreateElem(*old_node_idx, new),
                     ));
                 }
-                (Some((old_id, _)), Some((new_id, _))) => {
+                (Some(old_id), Some(new_id)) => {
                     if new_id != old_id {
                         patches.push(Patch::SpecialAttribute(
                             PatchSpecialAttribute::CallOnCreateElem(*old_node_idx, new),
@@ -154,13 +162,42 @@ fn diff_recursive<'a, 'b>(
                 (Some(_), None) | (None, None) => {}
             };
 
+            match (
+                old_element.special_attributes.on_remove_element_key(),
+                new_element.special_attributes.on_remove_element_key(),
+            ) {
+                (Some(_), None) => {
+                    patches.push(Patch::SpecialAttribute(
+                        PatchSpecialAttribute::CallOnRemoveElem(*old_node_idx, old),
+                    ));
+                }
+                (Some(old_id), Some(new_id)) => {
+                    if old_id != new_id {
+                        patches.push(Patch::SpecialAttribute(
+                            PatchSpecialAttribute::CallOnRemoveElem(*old_node_idx, old),
+                        ));
+                    }
+                }
+                _ => {}
+            }
+
             let old_elem_has_events = old_element.events.has_events();
             let new_elem_has_events = new_element.events.has_events();
 
             if !old_elem_has_events && new_elem_has_events {
-                patches.push(Patch::SetEventsId(*old_node_idx));
+                patches.push(Patch::SetEventsId {
+                    old_idx: *old_node_idx,
+                    new_idx: *new_node_idx,
+                });
             } else if old_elem_has_events && !new_elem_has_events {
                 patches.push(Patch::RemoveEventsId(*old_node_idx));
+            } else if old_elem_has_events && new_elem_has_events {
+                if old_node_idx != new_node_idx {
+                    patches.push(Patch::SetEventsId {
+                        old_idx: *old_node_idx,
+                        new_idx: *new_node_idx,
+                    });
+                }
             }
 
             generate_patches_for_children(
@@ -327,10 +364,10 @@ fn generate_patches_for_children<'a, 'b>(
 ///
 /// Along the way we also push patches to remove all tracked events for deleted nodes
 /// (if they had events).
-fn process_deleted_old_node_child(
-    old_node: &VirtualNode,
+fn process_deleted_old_node_child<'a>(
+    old_node: &'a VirtualNode,
     cur_node_idx: &mut u32,
-    patches: &mut Vec<Patch>,
+    patches: &mut Vec<Patch<'a>>,
 ) {
     *cur_node_idx += 1;
     if let VirtualNode::Element(element_node) = old_node {
@@ -338,6 +375,16 @@ fn process_deleted_old_node_child(
             patches.push(Patch::RemoveAllManagedEventsWithNodeIdx(*cur_node_idx));
         }
 
+        if element_node
+            .special_attributes
+            .on_remove_element_key()
+            .is_some()
+        {
+            patches.push(Patch::SpecialAttribute(
+                PatchSpecialAttribute::CallOnRemoveElem(*cur_node_idx, old_node),
+            ));
+        }
+
         for child in element_node.children.iter() {
             process_deleted_old_node_child(&child, cur_node_idx, patches);
         }
@@ -362,7 +409,7 @@ mod diff_test_case;
 mod tests {
     use super::*;
     use crate::event::EventName;
-    use crate::{html, wrap_closure, EventAttribFn, PatchSpecialAttribute, VText, VirtualNode};
+    use crate::{html, EventAttribFn, PatchSpecialAttribute, VText, VirtualNode};
     use std::collections::HashMap;
     use std::rc::Rc;
     use virtual_node::IterableNodes;
@@ -649,11 +696,14 @@ mod tests {
         let mut new = VirtualNode::element("div");
         set_on_create_elem_with_unique_id(&mut new, "150");
 
+        let mut expected = VirtualNode::element("div");
+        set_on_create_elem_with_unique_id(&mut expected, "150");
+
         DiffTestCase {
             old,
             new,
             expected: vec![Patch::SpecialAttribute(
-                PatchSpecialAttribute::CallOnCreateElem(0, &VirtualNode::element("div")),
+                PatchSpecialAttribute::CallOnCreateElem(0, &expected),
             )],
         }
         .test();
@@ -687,16 +737,274 @@ mod tests {
         let mut new = VirtualNode::element("div");
         set_on_create_elem_with_unique_id(&mut new, "99");
 
+        let mut expected = VirtualNode::element("div");
+        set_on_create_elem_with_unique_id(&mut expected, "99");
+
         DiffTestCase {
             old,
             new,
             expected: vec![Patch::SpecialAttribute(
-                PatchSpecialAttribute::CallOnCreateElem(0, &VirtualNode::element("div")),
+                PatchSpecialAttribute::CallOnCreateElem(0, &expected),
             )],
         }
         .test();
     }
 
+    /// Verify that we push an on remove elem patch if the new node has the special attribute
+    /// and the old node does not.
+    #[test]
+    fn on_remove_elem_for_replaced_elem() {
+        let mut old = VirtualNode::element("div");
+        set_on_remove_elem_with_unique_id(&mut old, "150");
+
+        let expected = {
+            let mut old = VirtualNode::element("div");
+            set_on_remove_elem_with_unique_id(&mut old, "150");
+
+            old
+        };
+
+        let new = VirtualNode::element("span");
+
+        DiffTestCase {
+            old,
+            new,
+            expected: vec![
+                Patch::Replace {
+                    old_idx: 0,
+                    new_idx: 0,
+                    new_node: &VirtualNode::element("span"),
+                },
+                Patch::SpecialAttribute(PatchSpecialAttribute::CallOnRemoveElem(0, &expected)),
+            ],
+        }
+        .test();
+    }
+
+    /// Verify that we push on remove element patches for replaced children, their replaced
+    /// children, etc.
+    #[test]
+    fn on_remove_elem_for_replaced_children_recursively() {
+        let mut grandchild = VirtualNode::element("strong");
+        set_on_remove_elem_with_unique_id(&mut grandchild, "key");
+
+        let mut child = VirtualNode::element("em");
+        set_on_remove_elem_with_unique_id(&mut child, "key");
+
+        child.as_velement_mut().unwrap().children.push(grandchild);
+
+        let old = html! {
+            <div>
+                {child}
+            </div>
+        };
+
+        let expected_child = {
+            let mut grandchild = VirtualNode::element("strong");
+            set_on_remove_elem_with_unique_id(&mut grandchild, "key");
+
+            let mut child = VirtualNode::element("em");
+            set_on_remove_elem_with_unique_id(&mut child, "key");
+            child.as_velement_mut().unwrap().children.push(grandchild);
+
+            child
+        };
+        let expected_grandchild = {
+            let mut grandchild = VirtualNode::element("strong");
+            set_on_remove_elem_with_unique_id(&mut grandchild, "key");
+            grandchild
+        };
+
+        let new = VirtualNode::element("span");
+
+        DiffTestCase {
+            old,
+            new,
+            expected: vec![
+                Patch::SpecialAttribute(PatchSpecialAttribute::CallOnRemoveElem(
+                    1,
+                    &expected_child,
+                )),
+                Patch::SpecialAttribute(PatchSpecialAttribute::CallOnRemoveElem(
+                    2,
+                    &expected_grandchild,
+                )),
+                Patch::Replace {
+                    old_idx: 0,
+                    new_idx: 0,
+                    new_node: &VirtualNode::element("span"),
+                },
+            ],
+        }
+        .test();
+    }
+
+    /// Verify that we push on remove element patches for truncated children, their children,
+    /// etc.
+    #[test]
+    fn on_remove_elem_for_truncated_children_recursively() {
+        let mut grandchild = VirtualNode::element("strong");
+        set_on_remove_elem_with_unique_id(&mut grandchild, "key");
+
+        let mut child = VirtualNode::element("em");
+        set_on_remove_elem_with_unique_id(&mut child, "key");
+
+        child.as_velement_mut().unwrap().children.push(grandchild);
+
+        let old = html! {
+            <div>
+                <span></span>
+                // Gets truncated.
+                {child}
+            </div>
+        };
+
+        let new = html! {
+            <div>
+                <span></span>
+            </div>
+        };
+
+        let expected_child = {
+            let mut grandchild = VirtualNode::element("strong");
+            set_on_remove_elem_with_unique_id(&mut grandchild, "key");
+
+            let mut child = VirtualNode::element("em");
+            set_on_remove_elem_with_unique_id(&mut child, "key");
+            child.as_velement_mut().unwrap().children.push(grandchild);
+
+            child
+        };
+        let expected_grandchild = {
+            let mut grandchild = VirtualNode::element("strong");
+            set_on_remove_elem_with_unique_id(&mut grandchild, "key");
+            grandchild
+        };
+
+        DiffTestCase {
+            old,
+            new,
+            expected: vec![
+                Patch::TruncateChildren(0, 1),
+                Patch::SpecialAttribute(PatchSpecialAttribute::CallOnRemoveElem(
+                    2,
+                    &expected_child,
+                )),
+                Patch::SpecialAttribute(PatchSpecialAttribute::CallOnRemoveElem(
+                    3,
+                    &expected_grandchild,
+                )),
+            ],
+        }
+        .test();
+    }
+
+    /// Verify that when patching attributes, if the old has an on remove element callback but the
+    /// new node does not, we call the on remove element callback.
+    ///
+    /// But only for that element, since the element's below it might not get removed from the
+    /// DOM.
+    #[test]
+    fn new_node_does_not_have_on_remove_elem() {
+        let old_child = on_remove_node_with_on_remove_child();
+        let mut old = html! {
+            <div>
+                {old_child}
+            </div>
+        };
+        set_on_remove_elem_with_unique_id(&mut old, "some-key");
+
+        let expected = {
+            let old_child = on_remove_node_with_on_remove_child();
+            let mut old = html! {
+                <div>
+                    {old_child}
+                </div>
+            };
+            set_on_remove_elem_with_unique_id(&mut old, "some-key");
+
+            old
+        };
+
+        let new_child = on_remove_node_with_on_remove_child();
+        let new = html! {
+            <div>
+                {new_child}
+            </div>
+        };
+
+        DiffTestCase {
+            old,
+            new,
+            expected: vec![Patch::SpecialAttribute(
+                PatchSpecialAttribute::CallOnRemoveElem(0, &expected),
+            )],
+        }
+        .test();
+    }
+
+    /// Verify that when patching attributes, if the old and new node are of the same tag type but
+    /// have different on remove element ID, a patch is pushed.
+    ///
+    /// But only for that element, since the element's below it might not get removed from the
+    /// DOM.
+    #[test]
+    fn different_on_remove_elem_id() {
+        let old_child = on_remove_node_with_on_remove_child();
+        let mut old = html! {
+            <div>
+                {old_child}
+            </div>
+        };
+        set_on_remove_elem_with_unique_id(&mut old, "start");
+
+        let expected = {
+            let old_child = on_remove_node_with_on_remove_child();
+            let mut old = html! {
+                <div>
+                    {old_child}
+                </div>
+            };
+            set_on_remove_elem_with_unique_id(&mut old, "start");
+
+            old
+        };
+
+        let new_child = on_remove_node_with_on_remove_child();
+        let mut new = html! {
+            <div>
+                {new_child}
+            </div>
+        };
+        set_on_remove_elem_with_unique_id(&mut new, "end");
+
+        DiffTestCase {
+            old,
+            new,
+            expected: vec![Patch::SpecialAttribute(
+                PatchSpecialAttribute::CallOnRemoveElem(0, &expected),
+            )],
+        }
+        .test();
+    }
+
+    /// Verify that if the old and new node have the same on remove element ID, no patch is pushed.
+    #[test]
+    fn same_on_remove_elem_id() {
+        let mut old = VirtualNode::element("div");
+        set_on_remove_elem_with_unique_id(&mut old, "same");
+
+        let mut new = VirtualNode::element("div");
+        set_on_remove_elem_with_unique_id(&mut new, "same");
+
+        DiffTestCase {
+            old,
+            new,
+            expected: vec![],
+        }
+        .test();
+    }
+
     /// Verify that if the old node and new node have the same dangerous_inner_html we do not push
     /// an SetDangerousInnerHtml patch.
     #[test]
@@ -781,7 +1089,51 @@ mod tests {
                         .into_iter()
                         .collect(),
                 ),
-                Patch::SetEventsId(0),
+                Patch::SetEventsId {
+                    old_idx: 0,
+                    new_idx: 0,
+                },
+            ],
+        }
+        .test();
+    }
+
+    /// Verify that we set the proper old and new node indices in the set events ID patch.
+    #[test]
+    fn uses_correct_new_node_idx_in_set_events_id_patch() {
+        let old = html! {
+            <div>
+                <em> <area /> </em>
+                <div></div>
+            </div>
+        };
+
+        let new = html! {
+            <div>
+                <span></span>
+                <div onclick=||{}></div>
+            </div>
+        };
+
+        DiffTestCase {
+            old,
+            new,
+            expected: vec![
+                Patch::Replace {
+                    old_idx: 1,
+                    new_idx: 1,
+                    new_node: &VirtualNode::element("span"),
+                },
+                Patch::AddEvents(
+                    3,
+                    vec![(&EventName::ONCLICK, &mock_event_handler())]
+                        .into_iter()
+                        .collect(),
+                ),
+                Patch::SetEventsId {
+                    old_idx: 3,
+                    new_idx: 2,
+                },
             ],
         }
         .test();
@@ -805,6 +1157,218 @@ mod tests {
         .test();
     }
 
+    /// Verify that if an earlier node in the tree was replaced, and a later node has events, all
+    /// nodes after it get their events ID increased based on the number of elements removed.
+    #[test]
+    fn resets_events_id_if_earlier_nodes_replaced() {
+        let old = html! {
+            <div>
+                // This node gets replaced
+                <span>
+                    <em>
+                        <area />
+                    </em>
+                </span>
+
+                <strong onclick=|| {}>
+                    <div></div>
+                    <a onclick=|| {}></a>
+                </strong>
+            </div>
+        };
+
+        let new = html! {
+            <div>
+                <div></div>
+
+                <strong onclick=|| {}>
+                    <div></div>
+                    <a onclick=|| {}></a>
+                </strong>
+            </div>
+        };
+
+        DiffTestCase {
+            old,
+            new,
+            expected: vec![
+                Patch::Replace {
+                    old_idx: 1,
+                    new_idx: 1,
+                    new_node: &VirtualNode::element("div"),
+                },
+                Patch::SetEventsId {
+                    old_idx: 4,
+                    new_idx: 2,
+                },
+                Patch::SetEventsId {
+                    old_idx: 6,
+                    new_idx: 4,
+                },
+            ],
+        }
+        .test();
+    }
+
+    /// Verify that if 5 earlier node were replaced replaced by 5 different nodes, we do not
+    /// reset the events ID for nodes that come after it since the total number of nodes has not
+    /// changed.
+    ///
+    /// This test should also cover cases where the same number of earlier nodes are
+    /// truncated / appended, since our implementation just checks whether or not the new node IDX
+    /// is equal to the old node IDX.
+    /// If not, then that node and every node after it needs its events ID reset
+    /// (if they have events).
+    #[test]
+    fn does_not_reset_events_id_if_earlier_node_replaced_by_same_number_of_nodes() {
+        let old = html! {
+            <div>
+                // This node gets replaced, but with the same number of nodes.
+                <span>
+                    <em>
+                        <area />
+                    </em>
+                </span>
+
+                <strong onclick=|| {}>
+                    <div></div>
+                    <a onclick=|| {}></a>
+                </strong>
+            </div>
+        };
+
+        let new = html! {
+            <div>
+                <div>
+                    <ul>
+                        <li> </li>
+                    </ul>
+                </div>
+
+                <strong onclick=|| {}>
+                    <div></div>
+                    <a onclick=|| {}></a>
+                </strong>
+            </div>
+        };
+
+        DiffTestCase {
+            old,
+            new,
+            expected: vec![Patch::Replace {
+                old_idx: 1,
+                new_idx: 1,
+                new_node: &html! {<div> <ul> <li> </li> </ul> </div>},
+            }],
+        }
+        .test();
+    }
+
+    /// Verify that if somewhere earlier in the tree there were child nodes truncated
+    /// (so the net number of earlier nodes decreased) we push a patch to set the later node's
+    /// events ID.
+    #[test]
+    fn resets_events_if_if_earlier_nodes_truncated() {
+        let old = html! {
+            <div>
+                // This node gets its children truncated.
+                <span>
+                    <em></em>
+                    <area />
+                </span>
+
+                <strong onclick=|| {}>
+                    <div></div>
+                    <a onclick=|| {}></a>
+                </strong>
+            </div>
+        };
+
+        let new = html! {
+            <div>
+                <span>
+                    <em></em>
+                </span>
+
+                <strong onclick=|| {}>
+                    <div></div>
+                    <a onclick=|| {}></a>
+                </strong>
+            </div>
+        };
+
+        DiffTestCase {
+            old,
+            new,
+            expected: vec![
+                Patch::TruncateChildren(1, 1),
+                Patch::SetEventsId {
+                    old_idx: 4,
+                    new_idx: 3,
+                },
+                Patch::SetEventsId {
+                    old_idx: 6,
+                    new_idx: 5,
+                },
+            ],
+        }
+        .test();
+    }
+
+    /// Verify that if somewhere earlier in the tree there were child nodes appended
+    /// (so the net number of earlier nodes increased) we push a patch to set the later node's
+    /// events ID.
+    #[test]
+    fn resets_events_if_if_earlier_nodes_appended() {
+        let old = html! {
+            <div>
+                // This node gets its children appended to.
+                <span>
+                    <em></em>
+                </span>
+
+                <strong onclick=|| {}>
+                    <div></div>
+                    <a onclick=|| {}></a>
+                </strong>
+            </div>
+        };
+
+        let new = html! {
+            <div>
+                <span>
+                    <em></em>
+                    <area />
+                </span>
+
+                <strong onclick=|| {}>
+                    <div></div>
+                    <a onclick=|| {}></a>
+                </strong>
+            </div>
+        };
+
+        DiffTestCase {
+            old,
+            new,
+            expected: vec![
+                Patch::AppendChildren {
+                    old_idx: 1,
+                    new_nodes: vec![(3, &VirtualNode::element("area"))],
+                },
+                Patch::SetEventsId {
+                    old_idx: 3,
+                    new_idx: 4,
+                },
+                Patch::SetEventsId {
+                    old_idx: 5,
+                    new_idx: 6,
+                },
+            ],
+        }
+        .test();
+    }
+
     /// Verify that if we previously had events but we no longer have any events we push a patch
     /// to remove the events ID.
     #[test]
@@ -864,7 +1428,7 @@ mod tests {
     /// We ensure that this event removal patch should come before the patch to replace the node,
     /// so that we don't accidentally remove events that were for the node that replaced it.
     #[test]
-    fn remove_tracked_events_if_replaced() {
+    fn remove_all_tracked_events_if_replaced() {
         let mut old = VElement::new("div");
         old.events.insert(oninput_name(), mock_event_handler());
 
@@ -950,10 +1514,14 @@ mod tests {
         node.as_velement_mut()
             .unwrap()
             .special_attributes
-            .on_create_elem = Some((
-            on_create_elem_id.into(),
-            wrap_closure(|_: web_sys::Element| {}),
-        ));
+            .set_on_create_element(on_create_elem_id, |_: web_sys::Element| {});
+    }
+
+    fn set_on_remove_elem_with_unique_id(node: &mut VirtualNode, on_remove_elem_id: &'static str) {
+        node.as_velement_mut()
+            .unwrap()
+            .special_attributes
+            .set_on_remove_element(on_remove_elem_id, |_: web_sys::Element| {});
     }
 
     fn set_dangerous_inner_html(node: &mut VirtualNode, html: &str) {
@@ -963,6 +1531,25 @@ mod tests {
             .dangerous_inner_html = Some(html.to_string());
     }
 
+    /// Return a node that has an on remove element function.
+    ///
+    /// This node has a child that also has an on remove element function.
+    ///
+    /// <div>
+    ///   <div></div>
+    /// </div>
+    fn on_remove_node_with_on_remove_child() -> VirtualNode {
+        let mut child = VirtualNode::element("div");
+        set_on_remove_elem_with_unique_id(&mut child, "555");
+
+        let mut node = VirtualNode::element("div");
+        set_on_remove_elem_with_unique_id(&mut node, "666");
+
+        node.as_velement_mut().unwrap().children.push(child);
+
+        node
+    }
+
     fn mock_event_handler() -> EventHandler {
         EventHandler::UnsupportedSignature(EventAttribFn(Rc::new(Box::new(JsValue::NULL))))
     }
