@@ -4,7 +4,8 @@
 //!
 //! wasm-pack test --chrome --headless crates/percy-dom --test events
 
-use percy_dom::event::EVENTS_ID_PROP;
+use crate::testing_utilities::{create_mount, document, get_element_by_id, random_id};
+use percy_dom::event::{EventHandler, EventName, EventsByNodeIdx, ManagedEvent, EVENTS_ID_PROP};
 use percy_dom::prelude::*;
 use percy_dom::{Patch, PercyDom, VElement};
 use std::cell::{Cell, RefCell};
@@ -15,6 +16,8 @@ use wasm_bindgen_test::*;
 
 wasm_bindgen_test_configure!(run_in_browser);
 
+mod testing_utilities;
+
 /// Verify that the oninput event works.
 ///
 /// wasm-pack test --chrome --headless crates/percy-dom --test events -- input_event_works
@@ -195,7 +198,10 @@ fn patch_set_events_id() {
 
     let elem = node.create_dom_node(0, &mut events);
 
-    let patch = Patch::SetEventsId(0);
+    let patch = Patch::SetEventsId {
+        old_idx: 0,
+        new_idx: 99,
+    };
 
     percy_dom::patch(
         elem.clone(),
@@ -206,7 +212,43 @@ fn patch_set_events_id() {
     .unwrap();
 
     let events_id = js_sys::Reflect::get(&elem, &EVENTS_ID_PROP.into()).unwrap();
-    assert!(events_id.as_string().is_some());
+    let events_id = events_id.as_string().unwrap();
+    let events_id = events_id.trim_start_matches(&events.events_id_props_prefix().to_string());
+    assert_eq!(events_id, "99");
+}
+
+/// Verify that if we apply a patch to set the node's events_id, we move all of its existing events
+/// over to the new ID.
+///
+/// wasm-pack test --chrome --headless crates/percy-dom --test events -- set_events_id_moves_events
+#[wasm_bindgen_test]
+fn set_events_id_moves_events() {
+    let node: VirtualNode = html! {
+        <div id=random_id() onclick=||{}></div>
+    };
+    let mut events = EventsByNodeIdx::new();
+    events.insert_managed_event(
+        0,
+        EventName::ONCLICK,
+        ManagedEvent::Delegated(EventHandler::NoArgs(Rc::new(RefCell::new(|| {})))),
+    );
+
+    let elem = node.create_dom_node(0, &mut events);
+
+    let patch = Patch::SetEventsId {
+        old_idx: 0,
+        new_idx: 99,
+    };
+
+    percy_dom::patch(
+        elem.clone(),
+        &VirtualNode::text("..."),
+        &mut events,
+        &[patch],
+    )
+    .unwrap();
+
+    assert!(events.get_event_handler(&99, &EventName::ONCLICK).is_some());
 }
 
 /// Verify that when replacing a node we set the __events_id__ property using the new_idx, not
@@ -628,7 +670,6 @@ fn node_with_events(
 const START_TEXT: &'static str = "Start Text";
 const APPEND_TEXT_ONE: &'static str = "- append1";
 const APPEND_TEXT_TWO: &'static str = "- append2";
-const MOUNT_ID: &'static str = "mount";
 
 fn start_text() -> Rc<RefCell<String>> {
     Rc::new(RefCell::new(START_TEXT.to_string()))
@@ -654,22 +695,6 @@ fn assert_text_appended(text: &Rc<RefCell<String>>, append: &str) {
     );
 }
 
-fn document() -> web_sys::Document {
-    web_sys::window().unwrap().document().unwrap()
-}
-
-fn get_element_by_id(id: &str) -> web_sys::Element {
-    document().get_element_by_id(id).unwrap()
-}
-
-fn create_mount() -> web_sys::Element {
-    let mount = document().create_element("div").unwrap();
-    mount.set_id(MOUNT_ID);
-    document().body().unwrap().append_child(&mount).unwrap();
-
-    mount
-}
-
 fn send_input_event(id: &str) {
     send_event::<web_sys::HtmlInputElement>(id, &web_sys::InputEvent::new("input").unwrap());
 }
@@ -697,8 +722,3 @@ where
         .dispatch_event(&event)
         .unwrap();
 }
-
-// Tests share the same DOM, so IDs need to be unique across tests.
-fn random_id() -> &'static str {
-    Box::leak(Box::new(js_sys::Math::random().to_string()))
-}
